#!/bin/bash

###
### This skript is developed by GWDG. Licensed under the Apache License, Version 2.0 (see http://www.apache.org/licenses/LICENSE-2.0)
###
### tibor.kalman@gwdg.de
### benedikt.wegmann@gwdg.de
###

### Script for PowerFolder login CALLBACK API
### $Id: powerfolder-tibor-login.sh 131 2013-07-22 16:11:23Z pfadmin $

### Arguments:
### $@ = login benedikt.wegmann@gwdg.de DnceD3oTTWmNs74a5+g/gg false

###-----------------------------------------------------------------------------
### Init values
###
PROG=${0##*/}
###TODO: check what doesnt work here
###PROGDIR=${0%%/*}
PROGDIR="/home/powerfolder/login-callback"
MAINPID=$$
DEBUG="0"
###-----------------------------------------------------------------------------

###-----------------------------------------------------------------------------
### Config
###
FILE_LOGFILE="${0}.log"
FILE_REALM2POLICY="${PROGDIR}/gwdg_realm2policygroup"
FILE_POLICY2ATTRIBUTE="${PROGDIR}/gwdg_policy2attribute"
SCRIPT_SET_QUOTA="${PROGDIR}/powerfolder-tibor-quota.sh"
SCRIPT_SET_PERMISSION="${PROGDIR}/powerfolder-tibor-permission.sh"

### IMPORTANT: Please replace the values in the following variables with your PowerFolder Server URL, admin account and password for the admin account!
PFSADMIN="admin@example.com"
PFSPASS='password'
PFSURL="http://localhost:8080"
###-----------------------------------------------------------------------------


###-----------------------------------------------------------------------------
### log
###-----------------------------------------------------------------------------
log() {
    ###test -n "$@" && echo "$(date +'%Y-%m-%d %H:%M:%S') $@" >> ${LOGFILE}
    echo "$(date +'%Y-%m-%d %H:%M:%S') $@" >> ${FILE_LOGFILE}
}

###-----------------------------------------------------------------------------
### error
###-----------------------------------------------------------------------------
error() {
    local EXITCODE=${1}
    shift 
    log "[ERROR] $@"
    log "----- LOGIN END -----"
###    kill $MAINPID
    exit ${EXITCODE}
}

isReadable() {
    local FILENAME=${1}
    if [ ! -r ${FILENAME} ] ; then
	error 2 "File ${FILENAME} is not readable!"
    fi
}

isWriteable() {
    local FILENAME=${1}
    if [ ! -w ${FILENAME} ] ; then
	log "[WARN] File ${FILENAME} is not writeable!"
    fi
}

isExecutable() {
    local FILENAME=${1}
    if [ ! -x ${FILENAME} ] ; then
	error 2 "File ${FILENAME} is not executable!"
    fi
}

getUserRealm() {
    local USERNAME=${1}
    log "Trying to identify the policy for [${USERNAME}]"
    REALM=${USERNAME##*@}
    log "REALM for user [${USERNAME}]: [${REALM}]"
    echo "${REALM}"
}


getRealmPolicy() {
    local REALM=${1}
    POLICY="nothing"
    log "Trying to get the policy for realm ${REALM}"

    isReadable "${FILE_REALM2POLICY}"

    ###### Fields in the REALM2POLICY file:


    # Zeilenweise Policy-Datei einlesen
    IFS='
'
    for LINE in $(cat ${FILE_REALM2POLICY}); do
	LINE=$(echo ${LINE} | tr -s [:blank:] "\t")
	if [ "$(echo "${LINE}" | grep -E "^#|^[:blank:]*$")" ]; then
	    ### Kommentare beginnen mit '#', Zeile auslassen
	    continue
	else
	    PATTERN="$(echo "${LINE}" | cut -f 2 | tr -d [:blank:])"
	    echo "${REALM}" | grep -E "${PATTERN}" >/dev/null 2>&1
	    if [ "$?" -eq 0 ] ; then
		### we found a fitting policy, exiting the current loop before its normal ending
		POLICY=$(echo "${LINE}" | cut -f 1 | tr -d [:blank:])
		break
	    elif [ "$?" -eq 2 ] ; then
		log "[WARN] Error occured during grepping POLICY for realm [${REALM}]"
	    fi
	fi
    done

    log "POLICY for realm [${REALM}]: [${POLICY}]"
    echo "${POLICY}"
}


setPolicy() {
    local USERNAME=${1}
    local POLICY=${2}
    log "Trying to set the policy for ${USERNAME}"
    
    ### Policy for: quota
    isReadable "${FILE_POLICY2ATTRIBUTE}"
    ### File content:
    ### policygroup         attribute=value
    ### gwdg    quota=10
    QUOTA=`grep "${POLICY}" ${FILE_POLICY2ATTRIBUTE} | grep 'quota' | head -n 1 | cut -f2 -d'=' | tr -d [:blank:]`
    ### TODO: we just grep the first policy which fits
    ### TODO: check empty QUOTA
    setUserQuota ${USERNAME} ${QUOTA}

    ### Policy for: Security Permissions
    for PERMISSION in FolderCreate FolderRemove ChangePreferences ; do
	### Policy for: FolderCreate
	log "Security permission: [${PERMISSION}]"
	ISGRANT=`grep "${POLICY}" ${FILE_POLICY2ATTRIBUTE} | grep "${PERMISSION}" | head -n 1 | cut -f2 -d'=' | tr -d [:blank:]`	### returns with yes or no
	### TODO: we just grep the first policy which fits
	
	### grant or revoke
	if [ "${ISGRANT}" == "yes" ] ; then
	    setUserPermission ${USERNAME} "grant" "${PERMISSION}"
	elif [ "${ISGRANT}" == "no" ] ; then
	    setUserPermission ${USERNAME} "revoke" "${PERMISSION}"
	else
	    error 21 "setPolicy: unknown permission parameter [$ISGRANT]"
	fi

	### TODO: wenn $PERMISSION in der Policydatei nicht auftaucht, kommt hier die unknonw... Fehlermeldung auch
    done
}

checkUser() {
    local USERNAME=${1}
    log "Checking if user ${USERNAME} is marked as manually created"
    USERACCINFO=$(wget -O - --no-check-certificate --http-user="${PFSADMIN}" --http-password="${PFSPASS}" "${PFSURL}/api/accounts?action=getInfo&username=${USERNAME}" 2>/dev/null | tr -s '[:space:]' ' ')
    if echo "$USERACCINFO" | tr -s "[:space:]{}" " " | grep "SKIPLOGINCALLBACK" >/dev/null ; then
        log "User ${USERNAME} IS marked as manually created, NOT processing login-callback"
        return 1
    else
        log "User ${USERNAME} NOT marked as manually created, processing login-callback"
        return 0
    fi
}

setUserQuota() {
    local USERNAME=${1}
    local QUOTA_IN_GB=${2}
    log "Trying to set the quota [${QUOTA_IN_GB}] for [${USERNAME}]"

    ###/home/powerfolder/powerfolder-tibor-quota.sh benedikt.wegmann@gwdg.de 11
    isExecutable "${SCRIPT_SET_QUOTA}"
    ${SCRIPT_SET_QUOTA} "${USERNAME}" "${QUOTA_IN_GB}"
    SCRIPT_RC=$?

    ### Quota script return code: 
    ### 0 if everything OK, something else if anything went wrong
    if [ ${SCRIPT_RC} -ne 0 ] ; then
	error 11 "setUserQuota was NOT successfull, RC is [${SCRIPT_RC}]"
    else
	log "setUserQuota was successfull, RC is [${SCRIPT_RC}]"
    fi
}

setUserPermission() {
    local USERNAME=${1}
    local ACTION=${2}
    local PERMISSION=${3}
    log "Trying to set [${ACTION}] permission [${PERMISSION}] for [${USERNAME}]"

    ###/home/powerfolder/powerfolder-tibor-permission.sh benedikt.wegmann@gwdg.de grant FolderCreate
    isExecutable "${SCRIPT_SET_PERMISSION}"
    ${SCRIPT_SET_PERMISSION} "${USERNAME}" "${ACTION}" "${PERMISSION}"
    SCRIPT_RC=$?

    ### Permission script return code: 
    ### 0 if everything OK, something else if anything went wrong
    if [ ${SCRIPT_RC} -ne 0 ] ; then
	error 11 "setUserPermission was NOT successfull, RC is [${SCRIPT_RC}]"
    else
	log "setUserPermission was successfull, RC is [${SCRIPT_RC}]"
    fi
}



###-----------------------------------------------------------------------------
### M A I N
###-----------------------------------------------------------------------------

### we do not log every admin access
if [ "${2}" == "${PFSADMIN}" ] ; then
    exit 0
fi
### we log not-first-logins into an other logfile
if [ "${4}" == "false" ] ; then
    isWriteable "${FILE_LOGFILE}.notfirstlogin.log"
    echo "$(date +'%Y-%m-%d %H:%M:%S') PARAMS: $@" >> ${FILE_LOGFILE}.notfirstlogin.log
    exit 0
fi

isWriteable "${FILE_LOGFILE}"
log "----- LOGIN BEGIN -----"
log "PARAMS: [${@}]"

if [ $# -eq 0 ]; then
    error 1 "No arguments specified."
###    usage
fi



############################################################################
### Login Callback API description:
### http://www.powerfolder.com/wiki/Login_callback_API
### Parameter 	Description 	Example
### action 	Currently always "login" 	login
### Username 	The username of the account 	myaccount@example.com
### OID 	The internal ID of the account in database 	1CEC6CABA78049428FAD278FED1FCEED
### firstLogin 	If this is the first login of this user at the server 	false
############################################################################

ACTION="${1}"
USERNAME="${2}"
INTERNALID="${3}"
ISFIRSTLOGIN="${4}"


if [ "${ISFIRSTLOGIN}" == "false" -o "${ISFIRSTLOGIN}" == "FALSE" ] ; then
    log "User [${USERNAME}]: firstLogin=${ISFIRSTLOGIN}, nothing to do, exiting..."
    echo "Login processed for [${USERNAME}], first Login [${ISFIRSTLOGIN}]."
elif [ "${ISFIRSTLOGIN}" == "true" -o "${ISFIRSTLOGIN}" == "TRUE" ] ; then
    if checkUser "${USERNAME}" ; then
        USER_REALM=`getUserRealm ${USERNAME}`
        USER_POLICY=`getRealmPolicy ${USER_REALM}`
        setPolicy "${USERNAME}" "${USER_POLICY}"
    fi
    echo "Login processed for [${USERNAME}], first Login [${ISFIRSTLOGIN}]."
else
    error 5 "User [${USERNAME}]: unknown firstLogin parameter [${ISFIRSTLOGIN}], exiting..."
fi

log "----- LOGIN END -----"
