#!/bin/bash

###
### This skript is developed by GWDG. Licensed under the Apache License, Version 2.0 (see http://www.apache.org/licenses/LICENSE-2.0)
###
### tibor.kalman@gwdg.de
### benedikt.wegmann@gwdg.de
###

### PowerFolder "Extended Security Control" settings script
### Using the PF REST API for setting up FolderCreate and ChangePreferences for a PF user
### $Id: powerfolder-tibor-permission.sh 103 2012-12-06 16:03:46Z pfadmin $

# ./powerfolder-tibor-permission.sh bwegman1@gwdg.de grant FolderCreate
# ./powerfolder-tibor-permission.sh bwegman1@gwdg.de revoke FolderCreate
# ./powerfolder-tibor-permission.sh bwegman1@gwdg.de grant FolderRemove
# ./powerfolder-tibor-permission.sh bwegman1@gwdg.de revoke FolderRemove
# ./powerfolder-tibor-permission.sh bwegman1@gwdg.de grant ChangePreferences
# ./powerfolder-tibor-permission.sh bwegman1@gwdg.de revoke ChangePreferences

### wget --no-check-certificate --http-user="pfadmin@gwdg.de" --http-password='PaSsWoRd' "http://localhost:8080/api/accounts?action=grant&username=bwegman1@gwdg.de&permissionID=FolderCreate&permissionID=ChangePreferences"
### {"ID":"DnceD3oTTWmNs74a5+g/gg","username":"bwegman1@gwdg.de","message":"bwegman1@gwdg.de granted [FolderCreatePermission, ChangePreferencesPermission]"}

### wget --no-check-certificate --http-user="pfadmin@gwdg.de" --http-password='PaSsWoRd' "http://localhost:8080/api/accounts?action=revoke&username=bwegman1@gwdg.de&permissionID=FolderCreate&permissionID=ChangePreferences"
### {"ID":"DnceD3oTTWmNs74a5+g/gg","username":"bwegman1@gwdg.de","message":"bwegman1@gwdg.de revoked [FolderCreatePermission, ChangePreferencesPermission]"}

### IMPORTANT: Please replace the values in the following variables with your PowerFolder Server URL, admin account and password for the admin account!
PFSADMIN="admin"
PFSPASS="password"
PFSURL="http://localhost:8080"

### Variables passed by the calling script
USERNAME="${1}"
ACTION="${2}"
PERMISSION="${3}"


echo "----- PERMISSION ------" >> ${0}.log
echo "$(date +'%Y%m%d%H%M%S'): PARAMS:${@}" >> ${0}.log

### Some sanity checks, make sure parameters are valid...
if [ $# -ne 3 ]; then
    echo "ERROR: number of parameters must be 2" >> ${0}.log
    exit 1
fi
        
if [ -z ${USERNAME} ] ; then
    echo "ERROR: username is not set" >> ${0}.log
    exit 2
fi
if [ -z ${ACTION} ] ; then
    echo "ERROR: action is not set" >> ${0}.log
    exit 3
fi
if [ -z ${PERMISSION} ] ; then
    echo "ERROR: permission is not set" >> ${0}.log
    exit 4
fi

if [ ${ACTION} != "grant" -a ${ACTION} != "revoke" ] ; then
    echo "ERROR: action is [$ACTION], but can only be GRANT or REVOKE" >> ${0}.log
    exit 11
fi

### TODO: 3mal and funktioniert nicht
#if [ ${PERMISSION} != "FolderCreate" -a ${PERMISSION} != "FolderRemove" -a ${PERMISSION} != "ChangePermission" ] ; then
#    echo "ERROR: permission is [$PERMISSION], but can only be FolderCreate or FolderRemove or ChangePermission" >> ${0}.log
#    exit 12
#fi
### TODO: make it more robust


### Poor Man's debugger, tkalman; 20120118


wget -O - -a ${0}.log \
    --no-verbose \
    --no-check-certificate \
    --http-user="${PFSADMIN}" --http-password="${PFSPASS}" \
    "${PFSURL}/api/accounts?action=${ACTION}&username=${USERNAME}&permissionID=${PERMISSION}" 2>&1 >> ${0}.log

WGET_RC=$?

echo "$(date +'%Y%m%d%H%M%S'):WGET RC:${WGET_RC}" >> ${0}.log
###echo "----- QUOTA ------" >> ${0}.log

exit ${WGET_RC}
